package me.mrletsplay.nojs._test;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import me.mrletsplay.nojs.action.Action;
import me.mrletsplay.nojs.action.ActionData;
import me.mrletsplay.nojs.action.ActionEvent;
import me.mrletsplay.nojs.action.ActionHandler;
import me.mrletsplay.nojs.action.ActionResult;
import me.mrletsplay.nojs.component.Component;
import me.mrletsplay.nojs.component.impl.Button;
import me.mrletsplay.nojs.component.impl.Form;
import me.mrletsplay.nojs.component.impl.Group;
import me.mrletsplay.nojs.component.impl.IFrame;
import me.mrletsplay.nojs.component.impl.Link;
import me.mrletsplay.nojs.component.impl.Message;
import me.mrletsplay.nojs.component.impl.Select;
import me.mrletsplay.nojs.component.impl.Text;
import me.mrletsplay.nojs.component.impl.TextInput;
import me.mrletsplay.nojs.page.Page;
import me.mrletsplay.nojs.page.layout.GridLayout;
import me.mrletsplay.nojs.page.layout.HClampLayout;
import me.mrletsplay.simplehttpserver.http.request.HttpRequestContext;
import me.mrletsplay.simplehttpserver.http.request.urlencoded.UrlEncoded;

public class LinksPage implements Page {

	private Map<String, String> links = new LinkedHashMap<>();

	@Override
	public Component getContent() {
		Group linksGroup = new Group()
			.layout(GridLayout.ofColumns(1)
				.gap("4px"));

		for(Map.Entry<String, String> s : links.entrySet()) {
			linksGroup.add(
				new Form().add(
					new Link()
						.href(s.getValue())
						.text(s.getKey())
						.target(Link.Target.BLANK),
					new Button()
						.text("Delete")
				)
				.action(new Action("removeLink")
					.extraData(new ActionData().put("link", s.getKey())))
				.layout(new GridLayout()
					.columns("1fr", "auto")
					.gap("4px")
					.vAlignment(GridLayout.VAlignment.START))
			);
		}

		if(links.isEmpty()) {
			linksGroup.add(new Text().text("No links added yet"));
		}

		UrlEncoded query = HttpRequestContext.getCurrentContext().getRequestedPath().getQuery();
		if(query.has("error")) {
			linksGroup.add(new Message()
				.text((String) query.getFirst("error"))
				.type(Message.Type.ERROR));
		}

		return new Group()
			.add(
				new Group().add(
					linksGroup,
					new Form().add(
						new Text().text("Add a new link:"),
						new Group().add(
							new TextInput("linkName").placeholder("Link Name"),
							new TextInput("link").placeholder("Insert link here"),
							new Button().text("Add Link"),
							new Select().choice("Test", "Amogus")
						)
						.layout(GridLayout.ofColumns(1)
							.gap("4px"))
					)
					.layout(GridLayout.ofColumns(1).gap("4px"))
					.action(new Action("addLink"))
				)
				.add(new IFrame(new DynPage()).data(new UrlEncoded(Map.of("error", List.of("Sussy baka iframe")))))
				.layout(GridLayout.ofColumns(1).gap("10px"))
			)
			.layout(new HClampLayout("200px", "100%", "800px")
				.hAlignment(HClampLayout.HAlignment.CENTER));
	}

	@Override
	public String getPath() {
		return "/links";
	}

	@ActionHandler("addLink")
	public void addLink(ActionEvent event) {
		String linkName = event.getData("linkName");
		String link = event.getData("link");
		if(link == null || link.isBlank() || (!link.startsWith("http://") && !link.startsWith("https://"))
			|| linkName == null || linkName.isBlank()) {
			UrlEncoded data = new UrlEncoded();
			data.set("error", "Invalid link");
			event.setResult(ActionResult.refresh(data));
			return;
		}

		links.put(linkName.trim(), link.trim());
	}

	@ActionHandler("removeLink")
	public void removeLink(ActionEvent event) {
		String link = event.getData("link");
		links.remove(link);
	}

}
