package me.mrletsplay.nojs.component.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import me.mrletsplay.nojs.component.Component;
import me.mrletsplay.simplehttpserver.dom.html.HtmlElement;
import me.mrletsplay.simplehttpserver.dom.html.element.HtmlSelect;

public class Select implements Component {

	private List<Choice> choices;

	public Select() {
		this.choices = new ArrayList<>();
	}

	public Select choice(Choice choice) {
		this.choices.add(choice);
		return this;
	}

	public Select choice(String value, String label) {
		return choice(new Choice(value, label));
	}

	public Select choices(Choice... choices) {
		for(Choice c : choices) {
			if(c == null) continue;
			this.choices.add(c);
		}

		return this;
	}

	public Select choices(Collection<Choice> choices) {
		for(Choice c : choices) {
			if(c == null) continue;
			this.choices.add(c);
		}

		return this;
	}

	public Select choices(Map<String, String> valuesAndLabels) {
		for(var entry : valuesAndLabels.entrySet()) {
			choice(entry.getKey(), entry.getValue());
		}

		return this;
	}

	@Override
	public HtmlElement toHtml() {
		HtmlSelect select = HtmlElement.select();
		select.addClass("select");

		for(Choice c : choices) {
			select.addOption(c.getLabel(), c.getValue());
		}

		return select;
	}

	public static class Choice {

		private final String value;
		private final String label;

		public Choice(String value, String label) {
			this.value = value;
			this.label = label;
		}

		public String getValue() {
			return value;
		}

		public String getLabel() {
			return label;
		}

	}

}
