package me.mrletsplay.nojs.page;

import me.mrletsplay.nojs.action.ActionEvent;
import me.mrletsplay.nojs.action.ActionHandlers;
import me.mrletsplay.nojs.component.Component;
import me.mrletsplay.simplehttpserver.dom.css.CssElement;
import me.mrletsplay.simplehttpserver.dom.css.CssSelector;
import me.mrletsplay.simplehttpserver.dom.css.StyleSheet;
import me.mrletsplay.simplehttpserver.dom.html.HtmlDocument;
import me.mrletsplay.simplehttpserver.dom.html.HtmlElement;
import me.mrletsplay.simplehttpserver.http.HttpRequestMethod;
import me.mrletsplay.simplehttpserver.http.HttpStatusCodes;
import me.mrletsplay.simplehttpserver.http.document.DocumentProvider;
import me.mrletsplay.simplehttpserver.http.document.HttpDocument;
import me.mrletsplay.simplehttpserver.http.header.DefaultClientContentTypes;
import me.mrletsplay.simplehttpserver.http.request.HttpRequestContext;
import me.mrletsplay.simplehttpserver.http.request.urlencoded.UrlEncoded;
import me.mrletsplay.simplehttpserver.http.response.HtmlResponse;

public interface Page extends HttpDocument, ActionHandlers {

	public Component getContent();

	public String getPath();

	public default HtmlDocument toHtml() {
		HtmlDocument doc = new HtmlDocument();
		doc.addStyleSheet("/base.css");

		StyleSheet sheet = new StyleSheet();

		CssElement cssE = new CssElement(CssSelector.selectClass("ee"));
		cssE.setProperty("text", "red");
		sheet.addElement(cssE);

		doc.getHeadNode().appendChild(HtmlElement.style(sheet));

		doc.getBodyNode().appendChild(getContent().toHtml());

		return doc;
	}

	@Override
	public default void createContent() {
		HttpRequestContext ctx = HttpRequestContext.getCurrentContext();

		if(ctx.getClientHeader().getMethod() == HttpRequestMethod.POST) {
			// TODO: error handling
			UrlEncoded data = ctx.getClientHeader().getPostData().getParsedAs(DefaultClientContentTypes.URLENCODED);

			ActionEvent e = new ActionEvent(data);
			onAction(e);

			String path = ctx.getClientHeader().getPath().getDocumentPath();
			if(e.getResult().getUrl() != null) {
				path = e.getResult().getUrl();
			}

			if(e.getResult().getData() != null) {
				path += "?" + e.getResult().getData().toString();
			}

			ctx.redirect(HttpStatusCodes.SEE_OTHER_303, path);
			return;
		}

		ctx.respond(HttpStatusCodes.OK_200, new HtmlResponse(toHtml()));
	}

	public default void register(DocumentProvider provider) {
		String path = getPath();
		provider.register(HttpRequestMethod.GET, path, this);
		provider.register(HttpRequestMethod.POST, path, this);
	}

	public static void register(DocumentProvider provider, Page... pages) {
		for(Page page : pages) {
			page.register(provider);
		}
	}

}
